// ---------------------------------------------------------------------------
//
// FishRob40.PAS                                        Ulrich Mller 31.05.05
//                                                      PCS1700        Delphi4
// Robot-Methoden auf Basis von FishFace40.DCU
//
// ---------------------------------------------------------------------------

unit FishRob40;

interface

uses
  Windows, Messages, SysUtils, Classes, FishFace40;

const
  nullPos = 999;

type
  PosSatz = record
    maxPosition: LongInt;
    aktPosition: LongInt;
  end;

  TFishRobot = class;
  TFishPosition = procedure(Sender: TFishRobot; Pos1, Pos2,
                                                Pos3, Pos4: LongInt) of object;

  TFishRobot = class(TFishFace)
  private
    FOnFishPosition: TFishPosition;
    function  GetmaxPosition(MotorNr: LongInt): LongInt;
    procedure SetmaxPosition(MotorNr, Wert: LongInt);
    procedure Starten(MotNr, zielPos: LongInt);
    function  WoIst(i, Pos: LongInt): LongInt;
    function  GetaktPosition(MotorNr: LongInt): LongInt;
    procedure SetaktPosition(MotorNr, Wert: LongInt);

  public
    property OnFishPosition: TFishPosition read  FOnFishPosition
                                           write FOnFishPosition;

    property aktPosition[MotorNr: LongInt]: LongInt read  GetaktPosition
                                                    write SetaktPosition;
    property maxPosition[MotorNr: LongInt]: LongInt read  GetmaxPosition
                                                    write SetmaxPosition;

    procedure MoveHome;
    procedure MoveTo(P1: LongInt=nullPos; P2: LongInt=nullPos;
                     P3: LongInt=nullPos; P4: LongInt=nullPos);

    constructor Create;
  end;
implementation

var
  PosTab: array[1..8] of PosSatz;

function TFishRobot.GetaktPosition(MotorNr: LongInt): LongInt;
begin
  Result := PosTab[MotorNr].aktPosition;
end;

procedure TFishRobot.SetaktPosition(MotorNr, Wert: LongInt);
begin
  PosTab[MotorNr].aktPosition := Wert;
end;

function TFishRobot.GetmaxPosition(MotorNr: LongInt): LongInt;
begin
  Result := PosTab[MotorNr].maxPosition;
end;

procedure TFishRobot.SetmaxPosition(MotorNr, Wert: LongInt);
begin
  PosTab[MotorNr].maxPosition := Wert;
end;

constructor TFishRobot.Create;
begin
  inherited Create;
  PosTab[1].maxPosition := 180;
  PosTab[2].maxPosition := 100;
  PosTab[3].maxPosition := 80;
  PosTab[4].maxPosition := 24;
end;

procedure TFishRobot.MoveHome;
// --- Anfahren der Home-Position ----------------------
var i: LongInt;
begin
  for i := 1 to 4 do begin
    PosTab[i].aktPosition := 0;
    SetMotor(i, ftiLeft, ftiFull, nullPos);
    end;
    WaitForMotors(0, ftiM1, ftiM2, ftiM3, ftiM4);
end;

procedure TFishRobot.MoveTo(P1, P2, P3, P4: LongInt);
// --- Simultanes Anfahren einer vorgegebenen Position -----------
begin
// --- Starten aller Motoren, wenn nicht nullPos ---
  Starten(ftiM1, P1);
  Starten(ftiM2, P2);
  Starten(ftiM3, P3);
  Starten(ftiM4, P4);

// --- Warten auf Ready aller Motoren und laufende Positionsangabe ---
  while WaitForMotors(80, ftiM1, ftiM2, ftiM3, ftiM4) = ftiTime do
    if Assigned(FOnFishPosition) then FOnFishPosition(Self,
        WoIst(ftiM1, P1), WoIst(ftiM2, P2), WoIst(ftiM3, P3), WoIst(ftiM4, P4));

// --- Aktualisieren PosTab ------------------------------------------
  if P1 <> nullPos then if P1 > PosTab[1].aktPosition
                        then PosTab[1].aktPosition := P1 + GetCounter(2)
                        else PosTab[1].aktPosition := P1 - GetCounter(2);
  if P2 <> nullPos then if P2 > PosTab[2].aktPosition
                        then PosTab[2].aktPosition := P2 + GetCounter(4)
                        else PosTab[2].aktPosition := P2 - GetCounter(4);
  if P3 <> nullPos then if P3 > PosTab[3].aktPosition
                        then PosTab[3].aktPosition := P3 + GetCounter(6)
                        else PosTab[3].aktPosition := P3 - GetCounter(6);
  if P4 <> nullPos then if P4 > PosTab[4].aktPosition
                        then PosTab[4].aktPosition := P4 + GetCounter(8)
                        else PosTab[4].aktPosition := P4 - GetCounter(8);

// --- Abschlieende Positionsangabe ------------------------------------
  if Assigned(FOnFishPosition) then FOnFishPosition(Self,
                                                    PosTab[1].aktPosition,
                                                    PosTab[2].aktPosition,
                                                    PosTab[3].aktPosition,
                                                    PosTab[4].aktPosition); 
end;

procedure TFishRobot.Starten(MotNr, zielPos: Integer);
// --- Starten eines Motors im RobMode, wenn zielPos <> nullPos ---
var ct: LongInt;
begin
  if zielPos <> nullPos then begin
    if zielPos > PosTab[MotNr].maxPosition then
                                          zielPos := PosTab[MotNr].maxPosition;
    if zielPos > PosTab[MotNr].aktPosition then
      SetMotor(MotNr, ftiRechts, ftiFull, zielPos-PosTab[MotNr].aktPosition)
    else begin
      ct := PosTab[MotNr].aktPosition - zielPos;
      if ct > 0 then SetMotor(MotNr, ftiLinks, ftiFull, ct);
      end;
  end;
end;

function TFishRobot.WoIst(i, Pos: Integer): LongInt;
// --- Bestimmen der aktuellen Position --------------------------
var j: LongInt;
begin
  j := (i-1) * 2 + 2;
  if Pos = nullPos then Result := PosTab[i].aktPosition
  else if Pos > PosTab[i].aktPosition then Result := Pos - GetCounter(j)
                                      else Result := Pos + GetCounter(j);
end;

end.
